/*
	$Id: IDManager.java 309 2007-03-24 21:57:27Z syl $

	Copyright (C) 2006 Syllabub
	Maroontress Fast Software.
*/

package com.maroontress.sanctuary;

import java.util.HashMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import org.w3c.dom.Element;

/**
   ̻ҥޥ͡Ǥ
*/
public final class IDManager {

    /** ͣΥ󥹥 */
    private static IDManager theInstance;

    /** splintȤɽ */
    private static final String SPLINT_COMMENT_REGEX
        = "/\\*@(([^/]|[^*]/)*)@\\*/";

    /** splintȤִʸ */
    private static final String SPLINT_COMMENT_REPLACEMENT
        = "<span class=\"splint\">($1)</span>";

    /** ѥɤɽ */
    private static final String COMPOUND_REGEX
        = "((struct|union|enum) @[1-9][0-9]*)";

    /** ѥ.Фɽ */
    private static final String COMPOUND_MEMBER_REGEX
	= "((struct|union|enum) *)?[_A-Za-z]\\w*(\\.[_A-Za-z]\\w*)?";

    /** ưŪ˥󥯤оݤȤʤʸ */
    private static final String ANCHOR_REGEX
        = COMPOUND_REGEX + "|" + COMPOUND_MEMBER_REGEX;

    /** ֤̾ȴɤΥϥåޥå */
    private HashMap<String, Location> dictionary;

    /** û̷ɤȴɤΥϥåޥå */
    private HashMap<String, String> shortcut;

    /** ANCHOR_REGEX򥳥ѥ뤷ɽΥѥ */
    private Pattern anchorPattern;

    /** SPLINT_COMMENT_REGEX򥳥ѥ뤷ɽΥѥ */
    private Pattern splintCommentPattern;

    /** ǸΥˡID */
    private int num;

    /** HTMLΥǥ졼 */
    private Decorator htmlDecorator;

    /** TeXΥǥ졼 */
    private Decorator texDecorator;

    /**
       ɤ򽤾륤󥿥ե
    */
    private interface Decorator {
	/**
	   ϥѡ󥯤Υɤޤ

	   @param location ɤ°̾
	   @param s Υ
	   @param anchor 󥫡
	   @return 
	*/
	String getAnchoredString(final Location location,
				 final String s,
				 final String anchor);

	/**
	   פ줿ɤޤ

	   @param s 
	   @return 
	*/
	String getEscapedString(final String s);
    }

    /**
       󥹥󥹤ޤ

       @return ̻ҥޥ͡
    */
    public static IDManager getInstance() {
	if (theInstance == null) {
	    theInstance = new IDManager();
	}
	return theInstance;
    }

    /**
       ̻ҥޥ͡ޤ
    */
    private IDManager() {
	dictionary = new HashMap<String, Location>();
	shortcut = new HashMap<String, String>();
	anchorPattern = Pattern.compile(ANCHOR_REGEX);
	splintCommentPattern = Pattern.compile(SPLINT_COMMENT_REGEX);
	num = 0;
	htmlDecorator = new Decorator() {
	    public String getAnchoredString(final Location location,
					    final String s,
					    final String label) {
		return location.getHTMLAnchor(s, label);
	    }
	    public String getEscapedString(final String s) {
		return Html.safe(s);
	    }
	};
	texDecorator = new Decorator() {
	    public String getAnchoredString(final Location location,
					    final String s,
					    final String label) {
		return Tex.safe(label);
	    }
	    public String getEscapedString(final String s) {
		return Tex.safe(s);
	    }
	};
    }

    /**
       ˡIDޤ

       @return ˡID
    */
    public int getUniqueID() {
	return ++num;
    }

    /**
       ꤷ֤̾˥ɤϢդޤ

       @param location ֤̾ΰ
       @param keyword 
    */
    public void addKeyword(final Location location, final Strings keyword) {
	String name = keyword.getFull();
	if (name.equals("")) {
	    return;
	}
	dictionary.put(name, location);
	String label = keyword.getShorten();
	if (name == label) {
	    return;
	}
	shortcut.put(name, label);
    }

    /**
       ꤷ֤̾˥ɤϢդޤ

       @param location ֤̾ΰ
       @param keyword 
    */
    public void addKeyword(final Location location, final String keyword) {
	if (keyword.equals("")) {
	    return;
	}
	dictionary.put(keyword, location);
    }

    /**
       ǥ졼ɤޤ

       @param decorator ǥ졼
       @param packageName ʸ󤬽°ѥå̾
       @param s 
       @return 
    */
    private String getDecoratedString(final Decorator decorator,
				      final String packageName,
				      final String s) {
	Location location = dictionary.get(s);
	if (location == null) {
	    return decorator.getEscapedString(s);
	}
	if (location.getPackageName().equals(packageName)) {
	    String label = shortcut.get(s);
	    if (label == null) {
		label = s;
	    }
	    return decorator.getAnchoredString(location, s, label);
	}
	return decorator.getAnchoredString(location, s, s);
    }

    /**
       ǥ졼ʸޤ

       @param decorator ǥ졼
       @param packageName ʸ󤬽°ѥå̾
       @param s ʸ
       @return ʸ
    */
    public String getString(final Decorator decorator,
			    final String packageName, final String s) {
	String in = s;
	String out = "";

	while (in.length() > 0) {
	    Matcher m = anchorPattern.matcher(in);
	    if (!m.find()) {
		out += decorator.getEscapedString(in);
		break;
	    }
	    int start = m.start();
	    int end = m.end();
	    out += decorator.getEscapedString(in.substring(0, start));
	    out += getDecoratedString(decorator, packageName,
				      in.substring(start, end));
	    in = in.substring(end);
	}
	return out;
    }

    /**
       HTMLʸޤ

       @param s ʸ
       @param packageName ʸ󤬽°ѥå̾
       @return HTMLʸ
    */
    public String getHTML(final String packageName, final String s) {
	String out = getString(htmlDecorator, packageName, s);
	return splintCommentPattern.matcher(out)
	    .replaceAll(SPLINT_COMMENT_REPLACEMENT);
    }

    /**
       TeXʸޤ

       @param s ʸ
       @param packageName ʸ󤬽°ѥå̾
       @return TeXʸ
    */
    public String getTEX(final String packageName, final String s) {
	return getString(texDecorator, packageName, s);
    }

    /**
       ʸCμ̻ҤƬʸȤڤޤ

       @param c ʸ
       @return trueǤʤfalse
    */
    private boolean isPrimaryChar(final char c) {
	return (c == '_' || (c >= 'a' && c <= 'z') || (c >= 'A' && c <= 'Z'));
    }

    /**
       ʸCμ̻Ҥ2ʸܰʹߤʸȤڤޤ

       @param c ʸ
       @return trueǤʤfalse
    */
    private boolean isSecondaryChar(final char c) {
	return (isPrimaryChar(c) || (c >= '0' && c <= '9'));
    }

    /**
       ʸCμ̻ҤȤڤޤ

       @param s ʸ
       @return trueǤʤfalse
    */
    public boolean isCID(final String s) {
	if (!isPrimaryChar(s.charAt(0))) {
	    return false;
	}
	for (int k = 1, len = s.length(); k < len; ++k) {
	    if (!isSecondaryChar(s.charAt(k))) {
		return false;
	    }
	}
	return true;
    }

    /**
       ǤCμ̻ҤȤʤ°ͤޤ

       ͤʸξNodeException򥹥ޤ

       @param elem 
       @param attrName °̾
       @return °
    */
    public String getIDAttribute(final Element elem, final String attrName) {
	String s = elem.getAttribute(attrName);
	if (s.equals("")) {
	    String m = attrName + " attribute is empty.";
	    throw new NodeException(elem, m);
	}
	if (!isCID(s)) {
	    String m = "'" + s + "' is an invalid identifier.";
	    throw new NodeException(elem.getAttributeNode(attrName), m);
	}
	return s;
    }
}
