/*
	$Id: MethodMember.java 309 2007-03-24 21:57:27Z syl $

	Copyright (C) 2006 Syllabub
	Maroontress Fast Software.
*/

package com.maroontress.sanctuary;

import java.io.PrintWriter;
import org.w3c.dom.Element;

/**
   ¤Τޤ϶Τ˴ޤޤؿݥ󥿷ΥФǤ
*/
public class MethodMember extends Member {

    /** ؿΰ */
    private MethodParameters params;

    /** ؿν */
    private Modifiers modifiers;

    /** ؿ */
    private Return returnElem;

    /**
       ؿݥ󥿷ΥФޤ

       @param g 饹
       @param parentName °빽¤Τޤ϶Τ̾
       @param elem method
    */
    public MethodMember(final Class g, final String parentName,
			final Element elem) {
	super(g, parentName, "", elem);

	ChildrenFilter c = new ChildrenFilter(elem);
	params = new MethodParameters(
	    g, c.getDirectChildrenByTagName("param"),
	    c.getDirectChildrenByTagName("variableparam"));
	modifiers = new Modifiers(c.getDirectChildrenByTagName("modifier"));
	returnElem = new Return(g, c.getDirectChildrenByTagName("return"));
    }

    /**
       {@inheritDoc}
    */
    @Override public void printCode(final PrintWriter wr,
				    final String indent) {
	String pre = "(*";
	String post = ")";
	Parameter p = params.getParameter(0);
	String arg = (p == null) ? "void" : p.getCode();
	String s = returnElem.getLeftCode() + pre + getName() + post + "(";
	int len = s.length();
	char[] space = new char[len];
	for (int i = 0; i < len; ++i) {
	    space[i] = ' ';
	}
	String offset = new String(space);
	wr.print(indent + s + arg);
	for (int k = 1; (p = params.getParameter(k)) != null; ++k) {
	    wr.print(",\n" + indent + offset + p.getCode());
	}
	wr.println(")" + returnElem.getRightCode()
		   + modifiers.getCode() + ";");
    }

    /**
       {@inheritDoc}
    */
    @Override public void printSummary(final PrintWriter wr) {
	Group group = getGroup();
	String prefix = getParentName() + ".";
	String pre = "(*";
	String post = ")";
	String name = getName();
	Description desc = getDescription();

	wr.println(""
+ "<tr>\n"
+ "  <td align=\"right\" valign=\"top\" width=\"1%\">\n"
+ "    <font size=\"-1\">&nbsp;" + returnElem.getHTML(group)
+ "</font>\n"
+ "  </td>\n"
+ "  <td valign=\"top\">"
+ pre + "<a href=\"#" + prefix + name + "\"><b>" + name + "</b></a>"
+ post + "(" + params.getHTML(group) + ")<br>"
+ "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;"
+ desc.getBriefText() + "</td>\n"
+ "</tr>");
    }

    /**
       {@inheritDoc}
    */
    @Override public void printDetail(final PrintWriter wr) {
	Group group = getGroup();
	String prefix = getParentName() + ".";
	String pre = "(*";
	String post = ")";
	Parameter p = params.getParameter(0);
	String arg = (p == null) ? "void" : p.getHTML(group);
	String name = getName();
	Description desc = getDescription();

	wr.print(""
+ "<p><a name=\"" + prefix + name + "\"></a>"
+ "<font size=\"+1\"><b>" + name + "</b></font></p>\n"
+ "<table cellpadding=\"0\" cellspacing=\"0\">\n"
+ "  <tr>\n"
+ "    <td nowrap>" + returnElem.getHTML(group) + " "
                    + pre + "<b>" + name + "</b>" + post + "(</td>\n"
+ "    <td nowrap>" + arg);

	for (int k = 1; (p = params.getParameter(k)) != null; ++k) {
	    wr.print(""
+ ",</td>\n"
+ "  </tr>\n"
+ "  <tr>\n"
+ "    <td></td><td nowrap>" + p.getHTML(group));
	}
	wr.println(""
+ ")</td>\n"
+ "  </tr>\n"
+ "</table>\n"
+ "<dl>\n"
+ "<dd>" + desc.getText() + "</dd>\n");

	params.printDetail(wr);
	returnElem.printDetail(wr);

	wr.println(""
+ "</dl>");
    }

    /**
       {@inheritDoc}
    */
    @Override public void printTexDetail(final PrintWriter wr) {
	Group group = getGroup();
	Parameter p = params.getParameter(0);
	String texName = Tex.safe(getName());
	String arg, lf, indent;
	Description desc = getDescription();

	if (p == null) {
	    arg = "void";
	    lf = "";
	    indent = "";
	} else {
	    arg = p.getTEX(group);
	    lf = "\\\\\n";
	    indent = "\\Sindent ";
	}

	wr.println("");
	wr.println("\\Smember{" + texName + "}");
	wr.println("\\nopagebreak");
	wr.print("\\begin{Sindentation}{\\Sindentlength}{0pt}\n");

	// Syntax
	wr.println("\\begin{Ssyntax}%");
	wr.print(returnElem.getTEX(group) + " (*{\\bfseries " + texName + "})("
		 + lf + indent + arg);

	for (int k = 1; (p = params.getParameter(k)) != null; ++k) {
	    wr.println(",\\\\");
	    wr.println(indent);
	    wr.print(p.getTEX(group));
	}
	wr.println(")");
	wr.println("\\end{Ssyntax}");

	// Description
	wr.println("");
	wr.print(desc.getTexText());

	// Parameters
	params.printTexDetail(wr);

	// Returns
	returnElem.printTexDetail(wr);

	wr.print("\\end{Sindentation}");
    }
}
